@@ -14,8 +14,15 @@ class DynamoStreamEventHandler:
         """
         return "Records" in event
 
-    def __init__(self, event):
+    def __init__(self, event, attribute_path=None):
+        """
+        The constructor to handle incoming and interpret DynamoDB Stream events.
+        :param event: The incoming DyanmoDB stream event.
+        :param attribute_path: An array with the path to the content the caller is watching.
+                               Defaults to root if not provided.
+        """
         self.records = event.get("Records", [])
+        self.attribute_path = attribute_path or []
 
     def get_changed_record_information(self):
         """
@@ -98,10 +105,17 @@ class DynamoStreamEventHandler:
         diff_record = record.get("dynamodb")
         new_record = diff_record.get("NewImage")
         event_name = record.get("eventName")
-        modified_columns = list(new_record.keys())
         if event_name == "MODIFY":
             # Make this output consistent for unit testing purposes
             modified_columns = sorted(list(self._get_modified_columns_set(record)))
+        else:
+            # This is for the INSERT case. All the columns are new!
+            # Traverse down the path and scoop up all the keys at the end of travel.
+            new_record = dynamodb_util.loads(diff_record.get("NewImage"))
+            for attribute_name in self.attribute_path:
+                # Traverse down the full attribute path
+                new_record = new_record.get(attribute_name, {})
+            modified_columns = list(new_record.keys())
 
         return {
             "value": self._get_new_value(record),
@@ -127,7 +141,10 @@ class DynamoStreamEventHandler:
         * The `source_table_name`, for Properly, excludes the environment of the table.
         """
         diff_record = record.get("dynamodb")
-        old_record = diff_record.get("OldImage")
+        old_record = dynamodb_util.loads(diff_record.get("OldImage"))
+        for attribute_name in self.attribute_path:
+            # Traverse down the full attribute path
+            old_record = old_record.get(attribute_name, {})
         modified_columns = list(old_record.keys())
 
         return {
@@ -165,16 +182,20 @@ class DynamoStreamEventHandler:
         json_record = dynamodb_util.loads(new_record)
         return json_record
 
-    @staticmethod
-    def _get_modified_columns_set(record):
+    def _get_modified_columns_set(self, record):
         """
         Calculates which columns have been removed, added, or modified.
         :param record: A single DynamoDB record
         :return: A list of column names that have changed.
         """
         diff_record = record.get("dynamodb")
-        new_record = diff_record.get("NewImage")
-        old_record = diff_record.get("OldImage", {})
+
+        new_record = dynamodb_util.loads(diff_record.get("NewImage"))
+        old_record = dynamodb_util.loads(diff_record.get("OldImage", {}))
+        for attribute_name in self.attribute_path:
+            # Traverse down the full attribute path
+            new_record = new_record.get(attribute_name, {})
+            old_record = old_record.get(attribute_name, {})
 
         changed_key_set = set([])
         for key in new_record:
